Package: SimVitD
Type: Package
Title: Simulation Tools for Planning Vitamin D Studies
Version: 1.0.5
Authors@R: c( person( "Rebecca", "Mangan", email="manganre@tcd.ie", role = c( "aut" ) ), person(  "Jason", "Wyse", email = "wyseja@tcd.ie", role = c("aut","cre") ), person("Lina", "Zgaga", email="zgagal@tcd.ie", role= "aut") )
Description: Simulation tools for planning Vitamin D studies. Individual vitamin D status 
             profiles are simulated, modelling population heterogeneity in trial arms. 
             Exposures to infectious agents are generated, with infection depending on vitamin D status.  
License: GPL (>= 2)
Maintainer: Jason Wyse <wyseja@tcd.ie>
Imports: simpleboot
Suggests: parallel
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-07-16 15:30:46 UTC; jason
Author: Rebecca Mangan [aut],
  Jason Wyse [aut, cre],
  Lina Zgaga [aut]
Repository: CRAN
Date/Publication: 2026-07-16 16:20:02 UTC

importFrom("grDevices", "rainbow")
importFrom("graphics", "axis", "legend", "lines", "matplot", "points",
             "title", "par", "boxplot", "abline", "plot")
importFrom("stats", "coef", "rnorm", "rexp", "rgamma", "runif", "rbeta", "pnorm", "qnorm", "quantile", "sd", "var")
importFrom("simpleboot", "two.boot")


export(intensity.function)
export(vitd.curve)
export(exposure.levels)
export(infection.count)
export(power.calc)
export(infection.count.plot)
export(rr.curve.plot)
export(rr.profile.plot)

S3method(plot, vitd.curve)
S3method(summary, vitd.curve)
S3method(print, vitd.curve)

S3method(plot, exposure.levels)
S3method(summary, exposure.levels)
S3method(print, exposure.levels)

S3method(summary, infection.count)
S3method(print, infection.count)

S3method(plot, power.calc)
S3method(summary, power.calc)
S3method(print, power.calc)

plot.vitd.curve <- function( x, main = " ", xlab = " ", ylab = "25-hydroxyvitamin D", col=1:6, add = FALSE, ylim = NULL, ... ){
  time <- ( x[[1]] ) / ( 12 / pi )
  if( add ) 
  {
    matplot( time, t(x[[2]][[1]]), col=col, type="l", axes=FALSE, add=TRUE, ... )
    return()
  }
  if( is.null(ylim) ) ylim <- c(0,min(125,max(x[[2]][[1]]+10)))
  matplot( time, t(x[[2]][[1]]), type="l", col=col, main=main, xlab=xlab, ylab=ylab, axes=FALSE, ylim = ylim, ... )
  axis( side = 2,   ... )
  year <- ( length(x[[1]]) - 1 ) / (x$res * 12)
  
  if( year < 1 )
  {
    # this is the case of less than 1 year
    month.start <- floor( x$start * 12 )
    month.end <- floor( x$end * 12 )
    if( x$north.hemi ){
      if( 3 + month.end > 12 ){
        months <- c( month.abb[(3+month.start):12], month.abb[1:((3+month.end)%%12)] )
      }else{
        months <- month.abb[(3 + month.start):(3 + month.end)]
      }
    }else{
      if( 9 + month.start > 12 ){
        st <- (9 + month.start)%%12 
        months <- month.abb[st:((9+month.end)%%12)]
      }else{
        st <- 9 + month.start
        if( month.end > 4){ 
          months <- c( month.abb[st:12], month.abb[1:((9+month.end)%%12)] )
        }else{
          months <- c( month.abb[st:(9+month.end)])
        }
      }
    }
    axis( 1, at = seq(time[1], time[length(time)], length.out = floor(year*12) + 1 ),
          labels = months, ... )
  }else{
    if( x$north.hemi ){
      months <- c( month.abb[3:12], rep(month.abb, year - 1), month.abb[1:3] )
    }else{
      months <- c( month.abb[9:12], rep(month.abb, year - 1), month.abb[1:9] )
    }
    axis( 1, at = seq(time[1], time[length(time)], length.out = year*12 + 1 ),
          labels = months, ... )
  }
  
}

plot.exposure.levels <- function( x, col = "blue", ... ){
  if( .Device == "null device") stop("plot.vitd.curve must be called before plot.exposure.levels")
  n <- nrow(x[[1]])
  time <- x[[1]]
  for( i in 1:n ) points( time[i,], x[[2]][i,], col=col, ... )
}

# This function was taken from the wBoot package by Neil A Weiss 
boot.two.bca.wBoot <-
function(x, y, parameter, stacked = TRUE, variable = NULL, null.hyp = NULL,
                          alternative = c("two.sided","less","greater"), conf.level = 0.95,
                          type = NULL, R = 9999)
{
# require(simpleboot)
proportion <- mean
if (stacked)
  {
   variable <- all.names(substitute(x))
   if (length(variable)>1) variable <- variable[[3]]
   y <- as.factor(y)
   l <- levels(y)
   pop.1 <- l[1]
   pop.2 <- l[2]
   u <- x[y==l[1]]
   v <- x[y==l[2]]
   x <- u
   y <- v
  } 
 else
  {
   pop.1 <- all.names(substitute(x))
   if (length(pop.1)>1) pop.1 <- pop.1[[3]]
   pop.2 <- all.names(substitute(y))
   if (length(pop.2)>1) pop.2 <- pop.2[[3]]
  }
test <- !is.null(null.hyp)
m <- length(x)
n <- length(y)
statistic <- parameter
obs.1 <- statistic(x)
obs.2 <- statistic(y)
obs <- obs.1-obs.2
boot.statistic <- two.boot(x,y,statistic,R=R)
z <- c(boot.statistic$t)
bias <- signif(mean(z)-obs,digits=3)
percent.bias <- signif(100*abs(bias/obs),digits=3)
cl <- paste(100*conf.level,"%",sep="")
if (identical(alternative,c("two.sided","less","greater"))) alternative <- "two.sided"
z1 <- mean(z < obs)
z0 <- qnorm(z1)
u <- statistic(x)
v <- statistic(y)
d <- vector(length=m+n)
for (i in 1:m) {d[i] <- statistic(x[-i])-v}
for (j in 1:n) {d[m+j] <- u-statistic(y[-j])}
mean.d <- mean(d)
a0 <- -sum((d-mean.d)^3)/(6*(sum((d-mean.d)^2))^(3/2))
# FOR THE HYPOTHESIS TEST
if (test)
{
 rtp <- (sum((z < null.hyp))+(sum((z == null.hyp))+1)/2)/(R+1)
 b0 <- qnorm(rtp)
 c0 <- ((2+a0*b0-a0*z0)*z0-b0)/(1+a0*b0-a0*z0)
 p0 <- pnorm(c0); # P-value for a left-tailed test
 tc <- c("two.sided","less","greater")
 pc <- c(2*min(p0,1-p0),p0,1-p0)
 p <- signif(pc[tc==alternative],digits=3)
 pv <- c((p>=0.001)&(p<=0.999),(p<0.001),(p>0.999))
 pt <- c(p,"P < 0.001","P > 0.999")
 p.value <- pt[pv]
 ac <- c("not-equal","less-than","greater-than")
 alt <- ac[tc==alternative]
}
# FOR THE CONFIDENCE INTERVAL
tci <- c("two.sided","greater","less")
ti <- c("two-sided","lower-bound","upper-bound")
if (is.null(type)) type <- ti[tci==alternative]
a <- if (identical(type,"two-sided")) (1-conf.level)/2 else 1-conf.level
za <- qnorm(a,lower.tail=FALSE)
a1 <- pnorm(z0+(z0-za)/(1-a0*(z0-za)))
a2 <- pnorm(z0+(z0+za)/(1-a0*(z0+za)))
q <- signif(quantile(z,p=c(a1,a2)),digits=4) 
li <- c(paste("(",q[1],", ",q[2],")",sep=""),paste(q[1],"(LCB)"),paste(q[2],"(UCB)"));
CI <- li[ti==type];
lims <- list(q,q[1],q[2]);
llims <- lims[[c(1:3)[ti==type]]];
# FOR THE RESULTS
param <- all.names(substitute(parameter))
stat.name <- paste("diff.",all.names(substitute(parameter)),sep="")
if (!test) {alt <- NULL; p.value <- NULL; p <- NULL}
results <- 
 list(Stacked=stacked,Boot.values=z,Confidence.limits=llims,Parameter=param,
      Header=paste("RESULTS OF BCa BOOTSTRAP FOR",toupper(stat.name)),
      Variable=variable,Pop.1=pop.1,Pop.2=pop.2,n.1=m,
      n.2=n,Statistic=stat.name,Observed.1=obs.1,Observed.2=obs.2,Observed=obs,
      Replications=R,Mean=mean(z),SE=sd(z),Bias=bias,Percent.bias=percent.bias,
      Null=null.hyp,Alternative=alt,P.value=p.value,p.value=p,Level=cl,Type=type,
      Confidence.interval=CI)
#class(results) <- "boot.two"  # bootstrap, two sample.
results
}

power.calc.0 <- function( idx, n, ratio, N, test.type, sig.level,
                          pl, tr,
                          baseline, rel.risk,
                          rate, intensity.func, 
                          holding.time, lohi.vit, boot.rep, clt ){
  
  num.sims <- N
  
  p <- numeric( num.sims )
  eff.size <- numeric( num.sims ) 
  
  lower <- baseline
  upper <- lower * rel.risk
  intercept <- -4
  slope <- 0.1
  
  start <- tr$time[1] / 12
  end <- tr$time[length(tr$time)] / 12
  cross  <- tr$cross/pi
  years <- end - start
  time <- seq( start * pi, end * pi, length.out = years * 24 + 1 )
  
  n.treat <- floor( ratio * n )
  
  for( dataset in 1:num.sims )
  {
    
    placebo <- vitd.curve( n = n, type = pl$type, start = start, end = end, mu = pl$mu, amplitude = pl$amplitude,
                           dyn.dose.thresh = pl$dyn.dose.thresh, sd.mu = pl$sd.mu, sd.amplitude = pl$sd.amplitude,
                           sd.dyn.dose.thresh = pl$sd.dyn.dose.thresh, supp.dose = pl$supp.dose, supp.dose.rate = pl$supp.dose.rate,
                           weight = pl$weight, sd.weight = pl$sd.weight, min.thresh = pl$min.thresh, north.hemi = pl$north.hemi)
    
    placebo.levels <- exposure.levels( placebo, rate, intensity.func = intensity.func, start, end )
    
    x.pl <- infection.count( placebo.levels, baseline, rel.risk, holding.time, lohi.vit=lohi.vit )$count
    
    treat <- vitd.curve( n = n.treat, type = tr$type, start = start, end = end, mu = tr$mu, amplitude = tr$amplitude,
                         dyn.dose.thresh = tr$dyn.dose.thresh, sd.mu = tr$sd.mu, sd.amplitude = tr$sd.amplitude,
                         sd.dyn.dose.thresh = tr$sd.dyn.dose.thresh, supp.dose = tr$supp.dose, supp.dose.rate = tr$supp.dose.rate,
                         weight = tr$weight, sd.weight = tr$sd.weight, min.thresh = tr$min.thresh, north.hemi = tr$north.hemi)
    
    treat.levels <- exposure.levels( treat, rate,intensity.func = intensity.func, start, end )
    
    x.tr <- infection.count( treat.levels, baseline, rel.risk, holding.time, lohi.vit=lohi.vit )$count
    
    if( test.type == 'proportions' )
    {
      x.pl <- as.integer( x.pl > 0)
      x.tr <- as.integer( x.tr > 0)
    }
    
    if( clt )
    {
      efsz <- mean(x.pl) - mean(x.tr)
      zobs <- efsz / sqrt( var(x.pl)/n  + var(x.tr)/n.treat )
      p[dataset] <- 1 - pnorm(zobs)
      eff.size[dataset] <- efsz
    }else{
      boot.test <- boot.two.bca.wBoot( x.pl, x.tr, stacked=FALSE, mean, null.hyp=0, alternative="greater", R=boot.rep )
      p[dataset] <- boot.test$p.value
      eff.size[dataset] <- mean(x.pl) - mean(x.tr)
    }
  }
  
  power <- sum( p < sig.level ) / num.sims
  
  return( list( power=power, eff.size= mean(eff.size)) )
  
}

plot.power.calc <- function( x, col = "hotpink", lwd = 1.5, lty = 1,
                             ylab = NULL, x.legend = NULL, y.legend = NULL,
                             main.legend = "Risk scaling", legend.size = 1, target.power = NA, which=1L, ... ){
  
  if( x$ratio == 1 ) xlab <- "n" else xlab <- paste0("n [ n supplement is ",x$ratio,"*n]")
  mainti <- paste( x$curve.type[1], "vs", x$curve.type[2], ":", x$test.type)

  if( is.null(ylab) & which==1L ) ylab <- "power" else ylab <- "effect size" 
  
  if( is.null(x.legend) ) x.legend <- min(x$npergroup)
  if( is.null(y.legend) ) y.legend <- 1
  
  lnrr <- length( x$RR )
  
  summaryPOWER <- vector( lnrr, mode="list" )
  
  if( which == 1L ) tarr <- x$power else tarr <- x$eff.size
  
  for( i in 1:lnrr ) 
  {
    if( x$mc.error > 1 )
    {
      if( length(x$npergroup) > 1 )
      { 
        summaryPOWER[[i]] <- apply( tarr[i, , ], 2, summary )
      }else{ 
        summaryPOWER[[i]] <- matrix(nrow=6,ncol=length(x$npergroup))
        summaryPOWER[[i]][,1] <- as.vector(summary( tarr[i,,1]))
      }
    }else{
      summaryPOWER[[i]] <- matrix(nrow=6,ncol=length(x$npergroup))
      summaryPOWER[[i]][3,] <- tarr[i,1,]
    }
    
  }  
  
  if( lnrr > 1 & length( x$npergroup ) == 1 ){ 
    
    if( x$mc.error == 1 )
    {
      
      plot( x$RR, tarr[,1,1], type = "l", col = col, lwd = lwd, lty = lty,
            main = mainti, ylim = c(0,1), xlab = "Risk scaling", ylab = ylab )
      
    }else{
      
      # make a vector of the medians
      meds <- numeric(lnrr)
      for( i in 1:lnrr ) meds[i] <- summaryPOWER[[i]][3,1]
      
      plot( x$RR, meds, type = "l", col = col, lwd = lwd, lty = lty,
            main = mainti, ylim = c(0,1), xlab = "Risk scaling", ylab = ylab, xlim=c(min(x$RR)-.5, max(x$RR)+.5) )
      
      for( i in 1:lnrr ) boxplot( tarr[i, ,1], add=TRUE, at=x$RR[i], xaxt="n", yaxt="n", boxfill="white", boxcol=col, outline=FALSE, medcol=col, whisklty="solid", whisklwd=1.5, whiskcol=col, staplecol=col )
      
    }
    
  }else if( lnrr > 0 & length( x$npergroup ) > 1 ){
    
    colour <- rainbow( lnrr )
    
    plot( x$npergroup, summaryPOWER[[1]][3,], type = "l", col = col, lwd = lwd, lty = lty,
          main = mainti, ylim = c(0,1), xlab = xlab, ylab = ylab, xaxt="n", xlim=c( min(x$npergroup)-1, max(x$npergroup)+1 ) )
    axis( 1, at=x$npergroup, labels=x$npergroup)
    
    if( x$mc.error > 1 ) boxplot( x$power[1, ,], use.cols=TRUE, add=TRUE, at=x$npergroup, xaxt="n", yaxt="n", boxfill="white", boxcol=col, outline=FALSE, medcol=col, whisklty="solid", whisklwd=1.5, whiskcol=col, staplecol=col )
    
    if( lnrr > 1 )
    {
      for( i in 2:lnrr ){
        lines( x$npergroup, summaryPOWER[[i]][3,], col = colour[i], lwd = lwd, lty = lty )
        if( x$mc.error > 1 ) boxplot( x$power[i, ,], use.cols=TRUE, add=TRUE, at=x$npergroup, xaxt="n", yaxt="n", boxfill="white", boxcol=colour[i], outline=FALSE, medcol=colour[i], whisklty="solid", whisklwd=1.5, whiskcol=colour[i], staplecol=colour[i] )
      }
    }
    
    if( !is.na(target.power) ) abline(h = target.power, lty="dotted", lwd=1.5, col="ForestGreen")
    legend(x.legend, y.legend, legend = x$RR, col = c(col, colour[2:lnrr]), title = main.legend, lwd = lwd, cex = legend.size )
    
  }else if( length( x$RR ) == 1 & length( x$npergroup ) == 1 ){
    
    if( x$mc.error == 1 )
    {
      stop("Only one relative risk and n value- plotting is not sensible.")
    }else{
      boxplot( tarr[1, ,], use.cols=TRUE, add=FALSE, boxfill="white", boxcol=col, outline=FALSE, medcol=col, whisklty="solid", whisklwd=1.5, whiskcol=col, staplecol=col, xlab=xlab, ylab=ylab, xaxt="n", main = mainti )
      axis(1, at=1, labels=x$npergroup )
    }
  }
}

summary.vitd.curve <- function( object, ... ){ # have to put in if loop if treatment / placebo
  
  x <- object
  
  type <- x$type # test for type of vitamin D curve 3 - placebo/trad 4 - treatment

  cat("\tLength of study:",(length( x$time )-1)/(12*x$res),"years")
  cat("\n\tNumber of participants:",nrow(x$curves$curves.eval))
  cat("\n\tExpected 25OHD level:",round(x$mu),"nmol/L")
  
  if( type == "placebo" | type ==  "fixed-dose"){
    if( type == "fixed-dose" ) cat("\n\tSupplement 25OHD contribution:",round(x$supp.dose),"nmol/L")
  }else{
    cat("\n\tThreshold 25OHD level:",round(x$dyn.dose.thresh),"nmol/L")
  }
}

power.calc <- function( n, ratio=1, N = 500, test.type, sig.level = 0.05,
                        vitdcurves.placebo = NULL, vitdcurves.treatment = NULL,
                        baseline = 0.03, RR = 3,
                        rate = 1, intensity.func = intensity.function(), 
                        holding.time = 0, lohi.vit = c(10,70), clt = NULL, mc.error = 1, boot.rep = 9999, parallel = FALSE, num.cores = NULL, verbose=FALSE ){
  
  placebo.group <- vitdcurves.placebo
  treatment.group <- vitdcurves.treatment
  if( ratio < 1 ) stop("Argument ratio should not be less than 1.")
  if( sig.level < 0 | sig.level > 1 ) stop("Argument 'sig.level' is a significance level.")
  if( baseline < 0 ) stop("Argument 'baseline' is a probability and must be positive.")
  if( any(RR < 0 ) ) stop("Some entries of 'RR' are negative.")
  if( rate < 0 ) stop("Argument 'rate' must be positive.")
  if( holding.time < 0 ) stop("Argument 'holding.time' must be positive." )
  if( mc.error < 0 | mc.error != floor(mc.error) ) stop("Argument 'mc.error' must be a positive integer.")
  if( any( baseline*RR > 1 ) ) stop("Argument 'RR' scales 'baseline' to probabilities greater than 1.")
  if( !( test.type %in% c('proportions','count') )) stop("Argument 'test.type' must be one of 'proportions','count'.")
  if( (!is.null(placebo.group) &  !inherits( placebo.group, "vitd.curve" ) ) | (!is.null(treatment.group) & !inherits( treatment.group, "vitd.curve" ) ) ) stop("Arguments 'vitdcurves.placebo' and 'vitdcurves.treatment' must be of class 'vitd.curve'.")
  if( mc.error == 1 & parallel ) warning("Setting parallel equal to true will only impact scenarios where mc.error > 1.")
  
  if( verbose & parallel ) cat("Printing progress updates is not possible with parallel set to TRUE.")
  
  # put in a check here for sizes of n and print warning for clt approximation-- pass clt approx as a vector?
  clt <- if( !is.null(clt) )
  {
    if( length(clt) > 1 & length(clt) < length(n) ) stop("If argument clt is a vector, it must have the same length as n.")
    if( length(clt) == 1 )
    {
      if( clt==FALSE & any(n >= 35) ) message("Some n values seem large: setting clt to TRUE may give a faster approximation than bootstrapping for these values.")
      if( clt==TRUE & any(n < 35) ) message("Some n values seem small: a clt approximation may not give ")
      rep(clt,length(n))
    }
  }else{
    # determine automatically
    message("Power approximation (CLT or bootstrap) has been determined for each instance in n: these can be set using argument clt")
    ap <- rep( TRUE, length(n) )
    ap[ n < 35 ] <- FALSE
    ap
  }
  
  num.participants <- n
  num.sims <- N
  
  if( parallel & !requireNamespace("parallel", quietly=TRUE) )  stop("Package \"parallel\" needed for this parallel functionality to work. Please install it.", call. = FALSE)
  
  if( parallel & !is.null(num.cores) )
  {
    # make sure parallelisation is sensible...  
    if( num.cores >  mc.error ) num.cores <- mc.error
  }
  
  if( parallel & mc.error > 1 & is.null(num.cores) ) num.cores <- parallel::detectCores() - 1
  
  # make the clusters for parallel simulation
  if( parallel )
  {
    cl <- parallel::makePSOCKcluster( num.cores )
    parallel::clusterEvalQ(cl, library(SimVitD))
    parallel::clusterSetRNGStream( cl ) 
  }
  
  curve.type <- c(placebo.group$type, treatment.group$type)
  
  pow.sim <- array( dim=c( length(RR), mc.error, length(num.participants) ) )
  eff.size <- array( dim=c( length(RR), mc.error, length(num.participants) ) )
  
  for( iter in 1:length(num.participants) )
  {
    ni <- num.participants[iter]
    
    if( verbose & !parallel ) cat("Beginning run for",ni,"participants: ")
    
    for( iteration in 1:length(RR) )
    {
      
      rel.risk <- RR[ iteration ]
      
      if( verbose & !parallel ) cat("\n\t... with RR =", rel.risk )
      
      if( parallel ){
        # parallel implementation 
        
        powpara <- parallel::parLapplyLB( cl, 1:mc.error, power.calc.0, ni, ratio, num.sims, test.type, sig.level,
                                          placebo.group, treatment.group,
                                          baseline, rel.risk,
                                          rate, intensity.func, 
                                          holding.time, lohi.vit, boot.rep, clt[iter] )
        
        for( r in seq_along(powpara) ) pow.sim[ iteration, r, iter ] <- powpara[[ r ]]$power
        for( r in seq_along(powpara) ) eff.size[ iteration, r, iter ] <- powpara[[ r ]]$eff.size
        
      }else{
        # not parallel- execute in sequence
        for( r in 1:mc.error )
        {
          # need to incorporate the ratio into here...
          powpara <- power.calc.0( 0, ni, ratio, num.sims, test.type, sig.level,
                                   placebo.group, treatment.group,
                                   baseline, rel.risk,
                                   rate, intensity.func, 
                                   holding.time, lohi.vit, boot.rep, clt[iter] )
          pow.sim[ iteration, r, iter ] <- powpara$power
          eff.size[ iteration, r, iter ] <- powpara$eff.size
        }
        
      }
      
      
    }
    if( verbose & !parallel ) cat("\n")
  }
  
  if( verbose & !parallel ) cat("All runs complete.")
  
  if( parallel ) parallel::stopCluster(cl)
  
  
  y <- list( curve.type, test.type, baseline, RR, num.participants, mc.error, pow.sim, eff.size, ratio )
  names( y ) <- c( "curve.type", "test.type", "baseline", "RR", "npergroup", "mc.error", "power", "eff.size", "ratio" )
  class( y ) <- "power.calc"
  return( y )
  
}

########################################################################
# x is output from exposure.levels function x[[2]] is matrix of vitamin D levels at time of exposures
infection.count <- function( expos, baseline = 0.03, RR = 3, holding.time = 0, lohi.vit = c(10,70) ){
  
  x <- expos
  
  if( !inherits( x, "exposure.levels" ) ) stop("Argument 'expos' not of class 'exposure.levels'")
  
  if( RR < 1 ) stop( "RR must be > 1" )
  if( length(lohi.vit)  != 2 ) stop( "lohi.vit must be of form (low,high)" ) 
  if( lohi.vit[1] > lohi.vit[2] ) stop( "lohi.vit must be of form (low,high)" )
  
  ht.rate <- ( 1/holding.time ) * ( 52/pi ) # holding time rate 

  lower <- baseline 
  upper <- baseline*RR # upper asymptote for generalised logisitic function
  
  if( upper > 1 ) stop("Argument 'RR' scales 'baseline' to probability greater than 1.")
  
  lo <- lohi.vit[1]
  hi <- lohi.vit[2]
  
  tau <- 0.045*(hi-lo) # tau to control inflection points
  
  slope <- log( (hi-lo-tau)^2/tau^2 ) / (hi-lo)
  intercept <- log( (hi-lo-tau)/tau ) - slope * hi
  
  ## where is lower defined in this?
  
  n <- nrow( x[[2]] ) # number of participants in the study
  num.exp <- ncol( x[[2]] ) # greatest number of possible exposures
  
  prob <- glf( x[[2]], lower, upper, intercept, slope )
  
  risk <- prob / lower
  
  u <- matrix( runif( n*num.exp, 0, 1 ), nrow=n, ncol=num.exp )
  
  disease <- u < prob # 1 if diseased, 0 if not diseased
  if(holding.time > 0) disease <- holding.time( disease, x[[1]], ht.rate ) # function that adds holding time if infection
  
  count <- rowSums( disease , na.rm = TRUE )
  mean <- mean( count )

  y <- list( baseline, RR, lohi.vit, prob, risk, disease, count, mean )
  names( y ) <- c("baseline", "RR", "inflection", "probs", "relativerisk", "infection", "count", "mean")
  class( y ) <- "infection.count"
  return( y )
}

summary.exposure.levels <- function( object, ... ){
  x <- object
  # number of exposures per participants
  U <- apply( x$exposures, 1, function(z) !is.na(z) )
  l <- colSums(U)
  if( length(l) <= 10 )
    cat("\tNumber of exposures per participant is: ",l )
  else
    cat("\tNumber of exposures for first 10 participants is: ",l[1:10])
}

# expos is output from exposure levels
# infect is output from infection.count
rr.curve.plot <- function( expos, infect, idx=1, main = NULL, xlab = "25-hydroxyvitamin D",
                           ylab = "Risk scaling", col = "blue", ... ){
   
  x <- expos
  y <- infect
                          
  if( !inherits( x, "exposure.levels" ) ) stop("Argument 'expos' not of class 'exposure.levels'")
  if( !inherits( y, "infection.count" ) ) stop("Argument 'infect' not of class 'infection.count'")
  
  n <- nrow(x[[1]])
  
  if( length(idx) > 1 | floor(idx) != idx ) stop("Set idx to a single index from 1:n")
  
  
  lvl <- seq( 0, 120, 1 )
  
  lower <- y$baseline
  upper <- lower * y$RR
  
  lo <- y$inflection[1]
  hi <- y$inflection[2]
  
  tau <- 0.045*(hi-lo) # tau to control inflection points
  
  slope <- log( (hi-lo-tau)^2/tau^2 ) / (hi-lo)
  intercept <- log( (hi-lo-tau)/tau ) - slope * hi
  
  OR.curve <- glf( lvl, lower, upper, intercept, slope )
  
  plot( lvl, OR.curve, type = "l", xlab = xlab, ylab = ylab, axes = FALSE, ... )
  axis( 2, at = seq( lower, upper, length.out = 6 ), labels = seq( 1, y$RR, length.out = 6 ) )
  axis( 1, at = seq( 0, 120, 20 ), labels = seq( 0, 120, 20 ) )
  if( is.null(main) ) main <- paste0( expos$type, " group" )
  if( length(which) == 1 ) main <- paste0( main, ": participant ", idx)
  title( main = main, ... )
  
  for( i in idx ){
    points( x$levels[i,], y$probs[i,], col = col, ... )
  }
  
  for( i in idx ){
    index <- which( y$infect[i,] == 1 )
    points( x$levels[i,index], y$probs[i,index], pch = 20, col = "red", ... )
  }
  
}

intensity.function <- function( summer.rate = 0, winter.rate = 1, flu = TRUE ){

  if( summer.rate < 0 || summer.rate > 1 ) stop( "summer.rate must be value between 0 and 1" )
  if( winter.rate < 0 || winter.rate > 1 ) stop( "winter.rate must be value between 0 and 1" )

  if( flu ){
    
    intensity <- function(t)
    {
      intens <- numeric( length(t) )
      
      intens[ t <= pi/6 ] <- winter.rate
      
      a <- ( t - pi/6 )/( pi/12 )
      
      b <- floor(a) %% 12
      
      intens[ ( b < 4 ) & t > pi/6 ] <- summer.rate
      
      intens[ ( b >= 4 ) & t > pi/6  ] <- winter.rate
      
      return(intens)
      
    }
    
  }else{
    
    intensity <- function(t)
    {
      return( rep( winter.rate, length(t) ) )
    }
  }
  return( intensity )
}

# vitdcurves is output from vitd.curve
# expos is output from exposure levels
# infect is output from infection.count

rr.profile.plot <- function( x, expos, infect, idx = 1, ... )
{
  
  vitdcurves <- x
  
  if( !inherits( vitdcurves, "vitd.curve" ) ) stop("Argument 'vitdcurves' not of class 'vitd.curve'")
  if( !inherits( expos, "exposure.levels" ) ) stop("Argument 'expos' not of class 'exposure.levels'")
  if( !inherits( infect, "infection.count" ) ) stop("Argument 'infect' not of class 'infection.count'")
  if( length(idx) > 1 ) warning("Length of idx is greater than 1: only first element used")
  idx <- idx[1]
  
  par( mfrow = c(1,2) )
  m <- 12*vitdcurves$res + 1 # take one yr
  time <- ( vitdcurves$time[1:m] ) / ( 12 / pi )
  h <- max( vitdcurves$curve$curves.eval[idx,] ) + 10
  
  # plot vit d profile for single participant
  plot( time, vitdcurves$curve$curves.eval[ idx, 1:m] , type = "l", xlab = "Time",
        ylab = "25 Hydroxy Vitamin D", ylim = c(0, h), axes = FALSE )
  
  axis( 2, ... )
  months <- c( month.abb[3:12], month.abb[1:3] )
  axis( 1, at = seq( time[1], time[length(time)], length.out = 13 ), labels = months, ... )
  
  year <- max( which(expos$exposures[ idx, ]  < pi) )
  points( expos$exposures[idx, 1:year], expos$levels[idx,1:year], col = "blue", cex=1.3, ...  )
  
  index <- which( infect$infection[idx,1:year] == 1 )
  points( expos$exposures[idx, index], expos$levels[idx,index], pch = 20, cex = 1.5, col = "red" )
  
# plot relative risk curve
  lvl <- seq( 0, h - 10 , 1 )
  
  lower <- infect$baseline
  upper <- lower * infect$RR
  
  lo <- infect$inflection[1]
  hi <- infect$inflection[2]
  
  tau <- 0.045*(hi-lo) # tau to control inflection points
  
  slope <- log( (hi-lo-tau)^2/tau^2 ) / (hi-lo)
  intercept <- log( (hi-lo-tau)/tau ) - slope * hi
  
  OR.curve <- glf( lvl, lower, upper, intercept, slope )
  
  plot( OR.curve, lvl, type = "l", 
        xlab="Relative Risk", ylab="", ylim=c(0, h), axes = FALSE, ... )
  axis( 1, at = seq( lower, upper, length.out = 6 ), labels = seq( 1, infect$RR, length.out = 6 ) )
  
  points( infect$probs[idx,1:year], expos$levels[idx,1:year], cex = 1.3, col = "blue" )
  points( infect$probs[idx,index], expos$levels[idx,index], type = "p", pch = 20, cex = 1.5, col = "red" )
  
  par( mfrow = c(1,1) )
  
}

print.power.calc <- function( x, ... ){
  if( x$mc.error == 1 )
  {
    if( length(x$npergroup) > 1 & length(x$RR) > 1 )
    {
      A <- x$power[,1,]
      colnames( A ) <- paste0("n=",x$npergroup)
      rownames( A ) <- paste0("RR=",x$RR)
    }else{
      A <- x$power[,1,]
      if( length(x$npergroup) > 1 ) 
        names(A) <- paste0("n=",x$npergroup)
      else
        names(A) <- paste0("RR=",x$RR)
    }
  }else{
    pr <- vector( length(x$RR), mode="list" )
    names(pr) <- paste0("RR=",x$RR)
    for( i in 1:length(x$RR) )
    {
      if( length(x$npergroup) > 1){
        pr[[i]] <- apply( x$power[i,,], 2, summary ) 
        colnames( pr[[i]] ) <- paste0("n=",x$npergroup)
      }else{
        pr[[1]] <- x$power[i,,]
        names( pr[[i]] ) <- rep( paste0("n=",x$npergroup), length(pr[[i]]) )
      }
    }
  }
  cat( "\tPower" )
  if( x$mc.error == 1 )
  {
    if( length(x$npergroup) > 1 & length(x$RR) > 1 )
    {
      cat("\n")
      print( round(A, 3) )
    }else{
      if( length(x$npergroup) > 1 ) 
      {
        cat( paste0(" @ RR=",x$RR, "\n" ) )
        print( round(A,3) )
      }else{
        cat( paste0(" @ n=",x$npergroup, "\n")  )
        print( round(A,3) )
      }
    }
  }else{
    cat("\n\t")
    print( pr )
  }
  
  if( x$mc.error > 1 ) cat( "\n\tMonte Carlo reps: ", x$mc.error ) 
}

summary.power.calc <- function( object, ... ){
  x <- object
  cat( "\tGroups tested are", x$curve.type[1],"and",x$curve.type[2])
  cat( "\n\tTest type is", x$test.type )
  cat( "\n\tBaseline prevalence is", x$baseline )
  cat( "\n\tRelative Risk is", x$RR )
}

# expos output from exposure.levels
# infect output from infection.count
infection.count.plot <- function( expos, infect, pch = 20, cex = 1.5, col = "red" ){
  x <- expos
  y <- infect
  if( !inherits( x, "exposure.levels" ) ) stop("Argument x is not of class 'exposure.levels'")
  if( !inherits( y, "infection.count" ) ) stop("Argument y is not of class 'infection.count'") 
  n <- nrow(x[[1]]) # number of participants
  time <- x[[1]] #/ (12/pi)
  for( i in 1:n ){
    index <- which( y$infection[i,] == 1 ) # vector of infection times
    points( time[i,index], x[[2]][i,index], pch = pch, cex = cex, col = col )
  }
}

exposure.levels <- function( x, rate, intensity.func = intensity.function(), start = 0, end = 1 )
{
  if( !inherits(x, "vitd.curve") )
    stop("Argument 'x' is not of class 'vitd.curve'")
  
  n <- nrow(x[[2]][[1]]) # gives number of participants in the study
  L <- rate * ( 52 / pi ) # 52/pi is one exposure per week 
  #start <- x$time[1] * (pi/12)
  if( end <  start ) stop( "End time must be greater than start time" )

  num.exposures <- ceiling( 1.5 * L * end * pi )  # simulate a conservative amount to buffer at end

  eventtimes <- matrix( rexp( n * num.exposures, rate=L  ), ncol=n )
  eventtimes <- start*pi + apply( eventtimes, 2, cumsum )
  eventtimes[ eventtimes > end*pi ] <- NA
  
  prob.acc <- intensity.func( eventtimes )
  U <- runif( n * num.exposures )
  Z <- U < prob.acc
  eventtimes[ !Z ] <- NA
  
  eventtimes <- t(eventtimes)

  mu <- x$curves$mu
  amplitude <- x$curves$amplitude
  weights <- x$curves$weights
  
  if( x$type == "dynamic-dose" ) thresh <- x$curves$thresh else thresh <- rep(0,n)
  
  eventheights <- event.height.na( thresh, mu, amplitude, eventtimes, x$min.thresh, x$supp.dose, weights )

  y <- list( eventtimes, eventheights, x$type )
  names( y ) <- c("exposures", "levels", "type")

  class( y ) <- "exposure.levels"
  return( y )
}

#################################################################

# Functions to be used for Vitamin D Simulation

#################################################################
# Rate Function for nhpp that has zero in summer and 1 in winter
# allows for many years

intensity.extreme <- function(t)
{
  
  intens <- numeric(length(t))
  
  intens[ t < pi/6 ] <- 1
  
  a <- ( t - pi / 6 ) / ( pi / 12 )
  
  b <- floor(a) %% 12
  
  intens[ b < 4 & t >= pi/6 ] <- 0
  
  intens[ b >= 4 & t >= pi/6 ] <- 1
  
  return( intens )

}

#################################################################
# Function that returns n random cosine curves, amplitudes and heights
random.cosine <- function( n=1, k.H=1, k.A=1, k.T=1, H.0=45, A.0=35, T.0=50, time=NULL, cross=-1, delta=20, tau=10, shape1=1, shape2=1 )
{
  
 timemat <- matrix( rep(time, n), ncol=length(time),  byrow=TRUE )

 # convert k.A and A.0 into shape and rate
 A <- rgamma( n, shape = (A.0/k.A)^2, rate = A.0/k.A^2 )
 
 H <- rnorm( n, mean = H.0, sd = k.H )
 
 omega <- rbeta( n, shape1 = shape1, shape2 = shape2)
 
 thresh <- rgamma( n, shape=(T.0/k.T)^2, rate=T.0/k.T^2 )
 
 supp <- delta * ( omega  + 0.5 * (1-omega) * (1 + sin(2*timemat - 3*pi/2) ) )
 
 curves <- H + A * cos( 2 * timemat - pi ) + supp 
 
 U <- curves < tau
 
 curves <- (1-U) * curves + U * tau
 
 rm(U)
 
 Z <- curves - thresh
 
 Z <- Z >= 0
 
 curves <- Z * curves + ( 1 - Z ) * thresh 
 
 return( list( curves.eval=curves, mu=H, amplitude=A, weights = omega, thresh = thresh ) )  
 
}

#################################################################
# Input event times from nhpp and returns vitamin d levels
# for flat sine curves when there are na event times
event.height.na <- function( thresh, H, A, t, tau, dose, weights)
{
	# evaluate curves
  output <- H + A * cos( 2*t - pi ) + dose * ( weights + 0.5 * (1-weights) * ( 1 + sin( 2*t - 3*pi/2) ))  
  
  # threhold this at 10 minimum
  U <- output < tau
  
  # correct for threshold
  output <- (1-U) * output + U * tau
  
  # threshold
  Z <- output - thresh
  Z <- Z > 0

  output <- Z * output + (1 - Z) * thresh
  
  return( output )
}



#################################################################
# Generate generalised logisitic regression curve

glf <- function( x, l.asym, u.asym, intercept, slope )
{
  return( l.asym + (u.asym-l.asym)/(1 + exp(intercept+slope*x)) )
}

#################################################################
# Input matrix with binary disease (0, 1), matrix of event times,
# rate - ouput vector with binary disease including holding time

holding.time <- function( infect, t, rate)
{
  
  n <- nrow( infect )
  m <- ncol( infect )
  
  tinf <- t * (infect == TRUE) 
  w <- apply( tinf, 1, function(x) which( x > 0 ) )
  for( i in seq_along(w) )
  {
    wi <- w[[i]]
    j <- 1
    while( j < length(wi) )
    {
      hold <- rexp(1, rate=rate)
      twihold <- tinf[i,wi[j]] + hold
      k <- 1
      while( tinf[i,wi[j+k]] < twihold && (j+k) < (length(wi)+1) ) 
      {
        infect[i,wi[j+k]] <- FALSE
        k <- k+1
        if( j+k == length(wi) ) break
      }
      j <- j+k
    }
  }
  return( infect )
}

#################################################################
# Truncated exponential for fixed increase
# 
rtrexp <- function(n, lambda, ub)
{
  u <- runif(n)
  ub + log( exp(- lambda*ub) + (1-exp(- lambda* ub) ) * u ) / lambda
}

dtrexp <- function(x, lambda, ub)
{
  lambda * exp(-lambda*(ub-x)) / (1-exp(- lambda* ub) )
}

summary.infection.count <- function( object, ... ){
  x <- object
  cat( "\tBaseline prevalence is", x$baseline )
  cat( "\n\tRelative Risk is", x$RR )
  n <- length( x$count )
  if( n > 10 ) 
    cat( "\n\tNumber of infections for first 10 participants is: ", x$count[1:10] )
  else 
    cat( "\n\tNumber of infections per participant is: ", x$count )
  cat( "\n\tProportion of infected in group is ",  length( which(x$count >= 1) ) / length(x$count) )
  cat( "\n\tMean number of infections in group is ", sum(x$count) / length(x$count) )
}

vitd.curve <- function( n = 1, type = c("placebo","fixed-dose","dynamic-dose"), start = 0, end = 1,  
                        mu = 45, amplitude = 35, dyn.dose.thresh = 50, sd.mu = 5, sd.amplitude = 5, sd.dyn.dose.thresh = 5, 
                        supp.dose = 20, supp.dose.rate = Inf, weight = 1, sd.weight = 0, min.thresh = 10, north.hemi = TRUE, res = 40  )
{
  
  type <- type[1]
  if( !( type %in% c("placebo","fixed-dose","dynamic-dose") ) ) stop("Argument 'type' is not valid")

  if( any( c(start < 0, end < 0, end <= start ) ) ) stop("Arguments 'start' and 'end' must be positive with 'end' > 'start'")
  if( any( c( mu < 10, mu  < amplitude ) ) ) stop("Please check arguments mu and amplitude are sensible")
  if( any( c(sd.mu < 0, sd.amplitude < 0, sd.dyn.dose.thresh < 0 ) ) ) stop( "Arguments 'sd' must be positive " ) 
  if( supp.dose < 0 ) stop("Argument 'supp.dose' must be positive")
  
  cross <- .5*(start + end) # not used
  
  years <- end - start
  time1 <- seq( start*pi, end*pi, length.out=years*(12*res) + 1 ) # pi is one year
  cross1 <- cross*pi
  time <- ( time1 * (12/pi) ) # gives time in terms of months
  
  if( type == "placebo" ) 
  {
    supp.dose <- 0
    delta <- rep(supp.dose,n)
    shape1 <- 1 # irrelevant- does not contribute here
    shape2 <- 1
    sd.dyn.dose.thresh <- 0
  }else if( type == "fixed-dose"){
    delta <- rtrexp( n, supp.dose.rate, supp.dose)   # allow for variable uptake in supplementation
    #delta <- rep(supp.dose,n) 
    sd.weight <- if( sd.weight > 0 ) sd.weight else ( (weight == 0 || weight == 1) & sd.weight == 0 ) * 0 + ( (weight > 0 & weight < 1) & sd.weight == 0 ) *  1e-4
    var.weight <- sd.weight^2
    c0 <- weight*(1-weight)/var.weight - 1
    shape1 <- weight*c0
    shape2 <- (1-weight)*c0
    if( sd.weight == 0 )
    {
      if( weight == 1 ){ shape1 <- Inf; shape2 <- 1 }
      if( weight == 0 ){ shape1 <- 1; shape2 <- Inf }
    }
    sd.dyn.dose.thresh <- 0
  }else if( type == "dynamic-dose"){
    supp.dose <- 0
    delta <- rep(supp.dose,n)
    shape1 <- 1 # irrelevant- does not contribute here
    shape2 <- 1
  }
  
  if( sd.amplitude == 0 ) sd.amplitude <- 1e-4
  
  y <- list( time, random.cosine( n , k.A = sd.amplitude, k.H = sd.mu, k.T = sd.dyn.dose.thresh, H.0 = mu,
                                  A.0 = amplitude, T.0 = dyn.dose.thresh, time = time1, delta=delta, tau=min.thresh, shape1=shape1, shape2=shape2 ), 
             mu, amplitude, dyn.dose.thresh,
             sd.mu, sd.amplitude, sd.dyn.dose.thresh, type, supp.dose, supp.dose.rate, weight, sd.weight, min.thresh, north.hemi, res, start, end, res )
  names( y ) <- c( "time", "curves", "mu", "amplitude", "dyn.dose.thresh", "sd.mu", "sd.amplitude", "sd.dyn.dose.thresh",
                   "type", "supp.dose", "supp.dose.rate", "weight", "sd.weight", "min.thresh", "north.hemi", "res", "start", "end", "res" )
  
  class( y ) <- "vitd.curve"
  return( y )
  
}

\name{summary.infection.count}
\alias{summary.infection.count}
\title{Summary of a infection.count object}
\description{
  Print a summary of a infection.count object.
}
\usage{
  \method{summary}{infection.count}( object, \dots )
}
\arguments{
  \item{object}{An object of class \code{infection.count}.}
   \item{\dots}{Optional arguments to lower level functions.}
}

\author{Rebecca Mangan and Jason Wyse}

\name{summary.exposure.levels}
\alias{summary.exposure.levels}
\title{Summary of a vitd.curve object}
\description{
  Print a summary of a exposure.levels object.
}
\usage{
  \method{summary}{exposure.levels}( object, \dots )
}
\arguments{
  \item{object}{An object of class \code{exposure.levels}.}
   \item{\dots}{Optional arguments to lower level functions.}
}

\author{Rebecca Mangan and Jason Wyse}

\name{rr.curve.plot}
\alias{rr.curve.plot}
\title{Plotting the relative risk curve for vitamin D status.}
\description{
  Plot the relative risk curve for vitamin D showing times of exposure and whether a participant developed infection from exposure.}
  \usage{
    rr.curve.plot( expos, infect, idx = 1, main = NULL, xlab = "25-hydroxyvitamin D",
        ylab = "Risk scaling", col = "blue", \dots )
  }
  \arguments{
    \item{expos}{An object of class \code{exposure.levels}.}
    \item{infect}{An object of class \code{infection.count}.}
    \item{idx}{A vector of indexes of specific exposures to plot.}
    \item{main}{Main title for the plot.}
    \item{xlab}{A title for the x-axis.}
    \item{ylab}{A title for the y-axis.}
    \item{col}{Character, value for the colour of the points.}
    \item{\dots}{Additional arguments to \code{plot}.}
  }

\author{Rebecca Mangan and Jason Wyse}

\name{infection.count.plot}
\alias{infection.count.plot}
\title{Plotting the times of disease on the vitamin D curves.}
\description{
  Plot whether a participant was disease or not as points on top of the vitamin D curves plotted from \code{plot.vitd.curve}.
  }
  \usage{
    infection.count.plot( expos, infect, pch = 20, cex = 1.5, col = "red" )
  }
  \arguments{
    \item{expos}{An object of class \code{exposure.levels}.}
    \item{infect}{An object of class \code{infection.count}.}
    \item{pch}{Plotting symbol for the points.}
    \item{cex}{Standard graphical parameter.}
    \item{col}{Colour of the plotted points.}
  }

\author{Rebecca Mangan and Jason Wyse}

\name{print.power.calc}
\alias{print.power.calc}
\title{Summary of a power.calc object}
\description{
  Print a summary of a power.calc object.
}
\usage{
  \method{print}{power.calc}( x, \dots )
}
\arguments{
  \item{x}{An object of class \code{power.calc}.}
   \item{\dots}{Optional arguments to lower level functions.}

}

\author{Rebecca Mangan and Jason Wyse}

\name{SimVitD-package}
\alias{SimVitD-package}
\docType{package}
\title{Simulation tools for vitamin D studies.}
\description{
A suite of tools for statistical planning of vitamin D trials. 
}

\details{ 

The \code{SimVitD} package uses simulation to aid in statistical planning of vitamin D trials. The core elements of the simulation based tools involve 
  
  (i) simulation of an individual vitamin D staus profiles taking into account natural cyclic variation in summer and winter
  
  (ii) simulation of exposures to potential infections in an individual's envrionment
  
  (iii) simulation of the possibility of developing infection conditional on vitamin D status at exposure
  
  (iv) Monte Carlo estimation of power of detecting supplementation effects for a selection of different supplementation scheme and two classes of hypothesis test.
  
The fuctions \code{\link{vitd.curve}} to generate individual status profiles and \code{\link{exposure.levels}} to simulate exposures. The accompanying vignette provides a gateway into the simulation tools. 
}

\examples{
# simulate individual Vitamin D profiles
indprofiles <- vitd.curve( n=10, type="placebo" )

# make a plot of all these profiles
plot( indprofiles )

# intensity function for exposures to infection
intensfun <- intensity.function( summer.rate=0.1, winter.rate=0.9 )

# exposure times
expos <- exposure.levels( indprofiles, rate=2, intensfun, end=2 )

# plot of exposures on top of Vitamin D profiles
plot( expos )

# disease calculation
infect <- infection.count( expos )

# plot disease points on top of exposure points
infection.count.plot( expos, infect )
}

\author{Rebecca Mangan, Jason Wyse, Lina Zgaga}

\name{summary.vitd.curve}
\alias{summary.vitd.curve}
\title{Summary of a vitd.curve object}
\description{
  Print a summary of a vitd.curve object.
}
\usage{
  \method{summary}{vitd.curve}( object, \dots )
}
\arguments{
  \item{object}{An object of class \code{vitd.curve}.}
   \item{\dots}{Optional arguments to lower level functions.}

}

\author{Rebecca Mangan and Jason Wyse}

\name{plot.exposure.levels}
\alias{plot.exposure.levels}
\title{Plotting the exposures to infection on vitamin D status curves.}
\description{
  Plot the exposures to infection as points on top of the vitamin D curves plotted from \code{plot.vitd.curve}.
  }
  \usage{
    \method{plot}{exposure.levels}( x, col = "blue", ... )
  }
  \arguments{
    \item{x}{An object of class \code{exposure.levels}.}
    \item{col}{Character, value for the colour of the points.}
    \item{\dots}{Additional arguments to be passed to the \code{plot} function.}
  }

\author{Rebecca Mangan and Jason Wyse}

\name{print.vitd.curve}
\alias{print.vitd.curve}
\title{Print summary of a vitd.curve object}
\description{
  Print a summary of a vitd.curve object.
}
\usage{
  \method{print}{vitd.curve}( x, \dots )
}
\arguments{
  \item{x}{An object of class \code{vitd.curve}.}
   \item{\dots}{Optional arguments to lower level functions.}

}

\author{Rebecca Mangan and Jason Wyse}

\name{infection.count}
\alias{infection.count}
\title{Generate whether a participant contracted a infection at exposure times.}
\description{
  Generates probabilities of developing infection, the relative risk and whether a participant becomes infected or not at exposure times.
}
\usage{
  infection.count( expos, baseline = 0.03, RR = 3, holding.time = 0, lohi.vit = c(10,70) )
}
\arguments{
  \item{expos}{An object of class \code{exposure.levels}.}

  \item{baseline}{Numeric, baseline prevalence of developing infection at any exposure time. Probability of developing infection when a participant has maximum (fully replete) vitamin D levels.}

  \item{RR}{Numeric, the relative risk of the study. The difference between becoming infected at highest and lowest vitamin D levels.}

  \item{holding.time}{Numeric, the mean number of weeks for after becoming infected when the participant can not become re-infected. Times are simulated independently from an exponential distribution with this mean.}
  
  \item{lohi.vit}{A vector of length 2 giving the level at which a participant is considered to have insufficient and sufficient vitamin D status levels.}
}

\value{

  \code{\link{infection.count}} returns an object of class \code{infection.count} that is a list.
  The list has the following slots.

  \item{baseline}{The baseline prevalence of disease.}

  \item{RR}{The relative risk of the study.}
  
  \item{inflection}{The inflection points of the relative risk curve.}

  \item{probs}{Matrix, the probability of developing infection at each exposure time for each participant.}

  \item{relativerisk}{Matrix, the relative risk of infection at each exposure time for each participant.}

  \item{infection}{Matrix, 1 if participant was infected at the corresponding exposure time, 0 if participant was not infected at exposure time.}

  \item{count}{Number of infections for each participant over the course of the study.}

  \item{mean}{The mean number of infections for the group over the course of the study.}

}

\author{Rebecca Mangan and Jason Wyse}

\name{rr.profile.plot}
\alias{rr.profile.plot}
\title{Plotting a vitamin D profile and relative risk curve.}
\description{
  Plot a vitamin D status profile for a single participant and the relative risk curve for vitamin D (with exposure times and whether a participant was infected at that exposure time) side by side.}
\usage{
rr.profile.plot( x, expos, infect, idx = 1, \dots )
}
\arguments{
  \item{x}{An object of class \code{vitd.curve}.}
  \item{expos}{An object of class \code{exposure.levels}.}
  \item{infect}{An object of class \code{infection.count}.}
  \item{idx}{Index of curve to plot.}
  \item{\dots}{Additional arguments to \code{plot}.}
}

\author{Rebecca Mangan and Jason Wyse}

\examples{
# individual profiles
indprofiles <- vitd.curve( n=10, type="placebo" )

# intensity function for exposures to infection
intensfun <- intensity.function( summer.rate=0.1, winter.rate=0.9 )

# exposure times
expos <- exposure.levels( indprofiles, rate=2, intensfun, end=2 )

# disease calculation
infect <- infection.count( expos )

# plot RR profile
rr.profile.plot( indprofiles, expos, infect )
}

\name{exposure.levels}

\alias{exposure.levels}

\title{Generate exposures to infectious agent and vitamin D status}

\description{
  \code{\link{exposure.levels}} creates a list of the exposures to an infectious agent and the status of vitamin D at the time of each exposure for each participant. The exposure times are simulated from a non homogeneous poisson process.
}

\usage{
  exposure.levels( x, rate, intensity.func = intensity.function(), start = 0, end = 1 )
}

\arguments{
  \item{x}{An object of class \code{vitd.curve}.}
  \item{rate}{Expected number of exposures per week, the rate at which exposures occur in an equivalent homogeneous Poisson process.}
  \item{intensity.func}{A function that takes time as sole argument and returns value between 0 and 1, scaling the rate argument. For example, exposures may be higher at certain times for seasonal infections.}
  \item{start}{Start time of the study in units of years.}
  \item{end}{End time of study in units of years.}
  }

\value{

  \code{\link{exposure.levels}} returns an object of class \code{exposure.levels} that is a list.
  The list has the following slots.

  \item{exposures}{The exposure times to infection for each participant. }
  \item{levels}{The vitamin D staus levels for each participant at the time of exposure to infection.}
  }
  
\author{Rebecca Mangan and Jason Wyse}

\name{plot.vitd.curve}
\alias{plot.vitd.curve}
\title{Plotting a vitd.curve object}
\description{
  Plot vitamin D status curves.
  }
\usage{
  \method{plot}{vitd.curve}( x, main = " ", xlab = " ", ylab = "25-hydroxyvitamin D", 
    col=1:6, add = FALSE, ylim = NULL, \dots )
}
\arguments{
  \item{x}{An object of class \code{vitd.curve}.}
  \item{main}{Main title for the plot.}
  \item{xlab}{A title for the x-axis.}
  \item{ylab}{A title for the y-axis.}
  \item{col}{A vector of colours for the plotted status curves.}
  \item{add}{If \code{TRUE}, add to current plotting window}
  \item{ylim}{Gives the plotting limits on tye y-axis.}
  \item{\dots}{Additional arguments to the \code{plot} function.}
}

\author{Rebecca Mangan and Jason Wyse}

\name{power.calc}
\alias{power.calc}
\title{Generate the power to detect the difference between two groups}
\description{
  generates a value for the power between two groups
}
\usage{
  power.calc( n, ratio=1, N = 500, test.type, sig.level = 0.05,
    vitdcurves.placebo = NULL, vitdcurves.treatment = NULL,
  	baseline = 0.03, RR = 3, rate = 1, intensity.func = intensity.function(), 
  	holding.time = 0, lohi.vit = c(10,70), clt = NULL, mc.error = 1, boot.rep = 9999, 
  	parallel = FALSE, num.cores = NULL, verbose=FALSE )
}
\arguments{
  \item{n}{Numeric, the number of participants in the control group.}
  
  \item{ratio}{Ratio greater than or equal to 1 giving size of treatment group as a multiple of \code{n}. Defaults to groups of equal size.}

  \item{N}{Number of simulations of the study to run.}

  \item{test.type}{Type of test to calculate the power, one of "count" or "proportions". }

  \item{sig.level}{Significance level used to test for a statistically significant difference between the groups.}
  
  \item{vitdcurves.placebo}{An object of class \code{vitd.curve}.}

  \item{vitdcurves.treatment}{An object of class \code{vitd.curve}.}

  \item{baseline}{Baseline prevalence of getting diseased at any exposure time. Probability of getting diseased when a participant has sufficient vitamin D levels.}

  \item{RR}{Fold risk difference between getting infection between the most deficient and most sufficient vitamin D levels.}

  \item{rate}{Expected number of exposures per week, the rate at which exposures occur in the equivalent homogeneous Poisson process.}

  \item{intensity.func}{Function taking time as sole argument and returns value between 0 and 1, input to nhpp function see poisson package.}

  \item{holding.time}{Expected number of weeks for the holding time.}
  
  \item{lohi.vit}{Inflection points of the relative risk curve used in \code{infection.count}.}
  
  \item{clt}{Logical or vector of logical values of same length as \code{n} indicating whether to use Central Limit Theorem approximation for the test when \code{TRUE}. Default is \code{NULL} and determination is automatic based on whether group size is less than 35 or not. }
  
  \item{mc.error}{Number of times to repeat the experiment at each \code{n} value to explore Monte Carlo error.}
  
  \item{boot.rep}{Number of bootstrap samples to carry out non-parametric tests of hypotheses.}
  
  \item{parallel}{Use parallel processing to carry out the simulations. This will parallelise over mc.error.}
  
  \item{num.cores}{Number of cores to exploit in parallel mode. Defaults to (cores available) - 1. }
  
  \item{verbose}{If \code{TRUE} print out regular updates on progression. Not available when parallel is set to \code{TRUE}.}
}

\value{

   \code{\link{power.calc}} returns an object of class \code{power.calc} that is a list.
  The list has the following slots.

  \item{test.type}{The type of study the power has been calculated on.}

  \item{baseline}{The baseline prevalence for disease.}

  \item{RR}{The relative risk of the study.}

  \item{npergroup}{The number of participants per group in the study.}
  
  \item{mc.error}{Number of repetitions of experiment to approximate Monte Carlo error.}

  \item{power}{A \code{length(RR)} x \code{mc.error} x \code{length(npergroup)} array of estimated power values. }
  
  \item{eff.size}{A \code{length(RR)} x \code{mc.error} x \code{length(npergroup)} array of estimated treatment effects (differences between groups).}

}

\author{Rebecca Mangan and Jason Wyse}


\examples{
# simulate placebo group
placebo <- vitd.curve( n = 10, type = "placebo" )

# simulate treatment group
treatment <- vitd.curve( n = 10, type = "dynamic-dose" )

# intensity function for exposures to infection
intensfun <- intensity.function( summer.rate = 0, winter.rate = 1 )
  
# calculate power: example only- run for much larger value of N
pow <- power.calc( n = c(10,20,30), 
        N = 10,  test.type = 'count', 
        vitdcurves.placebo = placebo, vitdcurves.treatment = treatment,
        baseline = 0.03, RR = c(2,4), rate = 1, intensity.func = intensfun, boot.rep=2000 )
            
### NOT RUN ###            
# approximate the Monte Carlo error in estimation of the power-- takes longer to run
#pow <- power.calc( n = c(10,20,30), 
#         N = 100, test.type = 'count', 
#         vitdcurves.placebo = placebo, vitdcurves.treatment = treatment,
#         baseline = 0.03, RR = c(2,3,4), rate = 1, holding.time = 2,
#         intensity.func = intensfun, mc.error = 10,
#         parallel = TRUE, num.cores = 10 )

# plot power curves
plot( pow, xlab = "n", x.legend = 10, y.legend = 1, main.legend = "Relative Risk" )
}

\name{intensity.function}
\alias{intensity.function}
\title{Generate an intensity function}
\description{
  Generates intensity function, function with time as sole argument.
}
\usage{
  intensity.function( summer.rate = 0, winter.rate = 1, flu = TRUE )
}
\arguments{
  \item{summer.rate}{Value bwteeen 0 and 1, the rate at which exposures occur in summer months.}

  \item{winter.rate}{Value between 0 and 1, the rate at which exposures occur in winter months.}

  \item{flu}{If \code{TRUE} define winter months as flu season (Sep-May in Northern Hemisphere, Mar-Nov in Southern Hemisphere). If \code{FALSE} have constant exposure rate all year.}

}

\value{

  \code{\link{intensity.function}} returns a function that takes
          time as sole argument and returns value between 0 and 1,
          the returned function acts as
          \code{intensity.func} for input to \code{\link{exposure.levels}}.
}


\author{Rebecca Mangan and Jason Wyse}

\name{plot.power.calc}
\alias{plot.power.calc}
\title{Plotting a power.calc object}
\description{
  Plot the power between two groups as the number of participants and the relative risk vary.
  }
  \usage{
    \method{plot}{power.calc}( x, col = "hotpink", lwd = 1.5, lty = 1, ylab = NULL, 
          x.legend = NULL, y.legend = NULL, main.legend = "Risk scaling", legend.size = 1, 
          target.power = NA, which = 1L, \dots )
  }
  \arguments{
    \item{x}{An object of class \code{power.calc}.}
    \item{col}{Colour of the line.}
    \item{lwd}{Line width.}
    \item{lty}{Line type.}
    \item{ylab}{Title for the y-axis.}
    \item{x.legend}{The x position of the legend. If not given, tries to default to a sensible value.}
    \item{y.legend}{The y position of the legend. If not given, tries to default to a sensible value.}
    \item{main.legend}{Title for the legend.}
    \item{legend.size}{Size of the legend.}
    \item{target.power}{Add a horizontal dotted line at a target power. A value of \code{NA} suppresses plotting.}
    \item{which}{Type of plot. Value 1 gives a plot of power, and value 2 plot of estimated effect size.}
    \item{\dots}{Additional arguments to be passed to the \code{plot} function.}
  }

\author{Rebecca Mangan and Jason Wyse}

\name{vitd.curve}
\alias{vitd.curve}
\title{Generate Vitamin D curves}
\description{
  Generates a vitamin D status profile curve for each individual in a group
}
\usage{
vitd.curve( n = 1, type = c("placebo","fixed-dose","dynamic-dose"), start = 0, end = 1,  
    mu = 45, amplitude = 35, dyn.dose.thresh = 50, sd.mu = 5, sd.amplitude = 5, 
    sd.dyn.dose.thresh = 5, supp.dose = 20, supp.dose.rate = Inf, weight = 1, 
    sd.weight = 0, min.thresh = 10, north.hemi = TRUE, res = 40 )
}
\arguments{
  \item{n}{Number of curves to simulate.}

  \item{type}{One of "placebo", "fixed-dose", "dynamic-dose".}

  \item{start}{Time in units of years when trial started.}

  \item{end}{Time in units of years when trial ended.}

  \item{mu}{The mean level of 25OHD in the trial arm around which there is cosine variation.}

  \item{amplitude}{Amplitude of cosine function describing variation around \code{mu}}

  \item{dyn.dose.thresh}{Threshold for the concentration-controlled scheme.}

  \item{sd.mu}{Standard deviation levels around mean; this is the standard deviation of H in vignette.}

  \item{sd.amplitude}{Standard deviation of the amplitude.}

  \item{sd.dyn.dose.thresh}{Standard deviation of the participant retained concentration in the concentration-controlled trial.}

  \item{supp.dose}{The 25OHD nmol/l equivalent for dosage in fixed-dose supplementation.}
  
  \item{supp.dose.rate}{Concentration parameter for fixed-dose scheme uptake. Large values imply that all participants derive the same equivalent.}
  
  \item{weight}{For fixed-dose supplementation, this is the expected value of the proportion of the dose which is always utilized.}
  
  \item{sd.weight}{Standard deviation of weight}
  
  \item{min.thresh}{The minimum detectable threshold of 25OHD. Defaults to 10 nmol/l.}
  
  \item{north.hemi}{Summer/winter months as in the Northern Hemisphere if \code{TRUE} and Southern Hemisphere if \code{FALSE}.}
  
  \item{res}{Resolution parameter for plotting of vitamin D curves.}
}

\value{
  \code{\link{vitd.curve}} returns an object of class \code{vitd.curve}. Curve parameter settings are returned. The slot \code{curves} give the curve specific parameters for each of the \code{n} generated curves. Additionally, \code{time} used for plotting gives time values passed to \code{plot.vitd.curve}.
}

\author{Rebecca Mangan and Jason Wyse}
